#include "ppc.h"
#include "m33.h"


PPC::PPC(float hfov, int _w, int _h) : w(_w), h(_h) {

	C = V3(0.0f, 0.0f, 0.0f);
	a = V3(1.0f, 0.0f, 0.0f);
	b = V3(0.0f, -1.0f, 0.0f);
	float hfovd = hfov / 180.0f * 3.1415926f;
	c = V3(-(float)w / 2.0f, (float)h / 2.0f, -(float)w / (2.0f * tan(hfovd / 2.0f)));

}


int PPC::Project(V3 P, V3 &pP) {

	M33 M;
	M.SetColumn(0, a);
	M.SetColumn(1, b);
	M.SetColumn(2, c);
	V3 q = M.Inverted()*(P - C);
	if (q[2] <= 0.0f)
		return 0;

	pP[0] = q[0] / q[2];
	pP[1] = q[1] / q[2];
	pP[2] = 1.0f / q[2];
	return 1;

}

V3 PPC::GetVD() {

	return (a^b).UnitVector();

}

void PPC::Rotate(V3 aDir, float theta) {

	a = a.RotateThisVector(aDir, theta);
	b = b.RotateThisVector(aDir, theta);
	c = c.RotateThisVector(aDir, theta);

}

float PPC::GetF() {

	return c*GetVD();

}

void PPC::PositionAndOrient(V3 Cn, V3 L, V3 upg) {

	V3 vdn = (L - Cn).UnitVector();
	V3 an = (vdn ^ upg).UnitVector();
	V3 bn = vdn ^ an;
	float f = GetF();
	V3 cn = vdn*f - an* (float)w / 2.0f - bn* (float)h / 2.0f;

	a = an;
	b = bn;
	c = cn;
	C = Cn;

}

