#include "scene.h"

#include "V3.h"
#include "M33.h"
#include "tm.h"

Scene *scene;

using namespace std;

#include <iostream>
#include <fstream>

Scene::Scene() {

	gui = new GUI();
	gui->show();

	int u0 = 16;
	int v0 = 40;
	int h = 400;
	int w = 600;
	fb = new FrameBuffer(u0, v0, w, h);
	fb->position(u0, v0);
	fb->label("first person");
	fb->show();
	fb->SetBGR(0xFF0000FF);
	fb->redraw();

	fb3 = new FrameBuffer(u0, v0, w, h);
	fb3->position(u0+fb->w+u0, v0);
	fb3->label("3rd person");
	fb3->show();
	fb3->SetBGR(0xFFFF0000);
	fb3->redraw();


	float hfov = 60.0f;
	ppc = new PPC(hfov, w, h);
	ppc3 = new PPC(hfov, w, h);
	ppc3->C = V3(-100.0f, 300.0f, 200.0f);

	tmsN = 1;
	tms = new TM[tmsN];
	tms[0].LoadBin("geometry/teapot1k.bin");
	tms[0].SetCenter(V3(0.0f, 0.0f, -150.0f));

	ppc3->PositionAndOrient(ppc3->C, tms[0].GetCenter(), V3(0.0f, 1.0f, 0.0f));

	gui->uiw->position(u0, v0 + fb->h + v0);

}

void Scene::Render(PPC *rppc, FrameBuffer *rfb) {

	rfb->SetBGR(0xFFFFFFFF);
	rfb->SetZB(0.0f);
	for (int tmi = 0; tmi < tmsN; tmi++) {
		tms[tmi].RenderWireFrame(rppc, rfb);
	}
	rfb->redraw();

}

void Scene::Render() {

	// first person
	Render(ppc, fb);
	// third person
	Render(ppc3, fb3);
	fb3->Render3DSegment(ppc3, ppc->C, ppc->C + ppc->GetVD()*100.0f, 
		V3(1.0f, 0.0f, 0.0f), V3(0.0f, 0.0f, 0.0f));
	fb3->redraw();

}

void Scene::DBG() {

	{
		Render();
		return;
	}

	{
		V3 cc(0.0f, 0.0f, -100.0f);
		float cubeSideLength = 30.0f;
		TM tm;
//		tm.SetAsAACube(cc, cubeSideLength);
		tm.LoadBin("geometry/teapot57k.bin");
		tm.SetCenter(V3(0.0f, 0.0f, -150.0f));
		int psize = 5;
		V3 Cn(ppc->C);
		V3 L = tm.GetCenter();
		V3 upg(0.0f, 1.0f, 0.0f);
		PPC ppc0(*ppc);
		int fN = 36;
//		fN = 1;
		for (int fi = 0; fi < fN; fi++) {
			fb->SetBGR(0xFFFFFFFF);
			fb->SetZB(0.0f);
//			tm.RenderAsPoints(psize, ppc, fb);
			tm.RenderWireFrame(ppc, fb);
			fb->redraw();
			Fl::check();
			float tstep = 0.1f;
			V3 tv = ppc->GetVD()*tstep;
			tv = ppc->a*tstep;
			V3 center = tm.GetCenter();
			ppc->Rotate(ppc->b.UnitVector()*-1.0f, 0.01f);
//			tm.Rotate(center, V3(0.0f, 1.0f, 0.0f), 0.1f);
//			ppc->C = ppc->C + tv;
			ppc->PositionAndOrient(Cn, L, upg);
			Cn = Cn + V3(0.0f, 5.0f, 0.0f);			
		}
		*ppc = ppc0;
		return;
	}
	{
		V3 P(0.0f, 0.0f, -100.0f);
		V3 pP;
		ppc->Project(P, pP);
		cerr << pP << endl;
		return;
	}

	{

		fb->SetBGR(0xFF000000);
		fb->SaveAsTiff("mydbg/blackImage.tif");
		fb->SetBGR(0xFF00FF00);
		fb->redraw();
		Fl::check();
		char c;
		cin >> c;
		fb->LoadTiff("mydbg/redImage.tif");
		fb->redraw();
		return;
	}

	{

		V3 p0(20.0f, 430.0f, 0.0f);
		V3 p1(520.0f, 331.0f, 0.0f);
		fb->SetBGR(0xFFFFFFFF);
		fb->DrawSegment(p0, p1, 0xFF000000);
		fb->redraw();
		return;

	}

	{
		V3 tlc(100.5f, 45.5f, 0.0f);
		V3 brc(300.5f, 145.5f, 0.0f);
		unsigned int col = 0xFF0000FF;
		unsigned int colDisk = 0xFFFFFF00;
		V3 center(114.1f, 300.1f, 0.0f);
		float r = 110.0f;
		int fN = 1000;
		float du = 1.f;
		for (int fi = 0; fi < fN; fi++) {
			fb->SetBGR(0xFFFFFFFF);
			fb->DrawAARectangle(tlc, brc, col);
//			fb->DrawDisk(center, r, colDisk);
			fb->redraw();
			Fl::check();
			tlc[0] += du;
			brc[0] += du;
			center[0] += du;
		}
		return;
	}

	{
		M33 m;
		V3 v(1.0f, 2.0f, 3.0f);
		m[0] = v;
		m[1] = v;
		m[2] = v;
		V3 ret = m*v;
		cerr << "mv: " << ret << endl;
		return;
	}
	{
		V3 v(1.0f, 2.0f, 3.0f);
		cerr << "v: " << v << endl;
		return;
	}

	cerr << "INFO: pressed DBG button on GUI" << endl;

	{
		V3 v0(1.0f, 2.0f, 3.0f);
		V3 v1(-1.0f, -2.0f, -3.0f);
		V3 v01 = v0 + v1;
		cerr << "v0 + v1: " << v01[0] << " " << v01[1] << " " << v01[2] << endl;
		return;
	}

	return;

	unsigned int col0 = 0xFF000000;
	unsigned int col1 = 0xFFFFFFFF;
	int csize = 100;
	fb->SetChecker(col0, col1, csize);
	fb->redraw();
}

void Scene::NewButton() {
	cerr << "INFO: pressed New button on GUI" << endl;
}

